/* $Id$ */
/* io-jpg.c by David Welton <davidw@linuxcare.com>.  Most of the code
   borrowed from Imlib */

#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <gmodule.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "gdk-pixbuf-save.h"

 /**
 * gdk_pixbuf_save_to_file:
 * @filename: Name of file to save.
 *
 * Saves pixbuf to a file.
 * 
 **/

static GdkPixbufSaveModule file_formats [] = {
    {"png", NULL, NULL},
    {"jpeg", NULL, NULL},
    {0, 0, 0}
};

/* actually load the image handler - gdk_pixbuf_get_module only get a */
/* reference to the module to load, it doesn't actually load it       */
/* perhaps these actions should be combined in one function           */
void
gdk_pixbufsave_load_module (GdkPixbufSaveModule *image_module)
{
    char *module_name;
    char *path;
    GModule *module;
    gpointer load_sym;
    
    g_return_if_fail (image_module->module == NULL);
    
    module_name = g_strconcat ("pixbufsave-", image_module->module_name, NULL);
    path = g_module_build_path (PIXBUFSAVE_LIBDIR, module_name);
    
    module = g_module_open (path, G_MODULE_BIND_LAZY);
    if (!module) {
	/* Debug feature, check in present working directory */
	g_free (path);
	path = g_module_build_path ("", module_name);
	module = g_module_open (path, G_MODULE_BIND_LAZY);
	
	if (!module) {
	    g_warning ("Unable to load module: %s: %s", path, g_module_error ());
	    g_free (module_name);
	    g_free (path);
	    return;
	}
	g_free (path);
    } else {
	g_free (path);
    }

    g_free (module_name);

    image_module->module = module;

    if (g_module_symbol (module, "image_save", &load_sym))
	image_module->save = load_sym;
}


gboolean gdk_pixbuf_save_to_file (const char *filename, GdkPixbuf *pixbuf, char *format, GdkPixbufInfo *imginfo)
{
    int i;
    FILE *f;
    GdkPixbufSaveModule *image_module = NULL;
    
    f = fopen (filename, "w");
    if (!f)
    {
	g_warning ("Cannot open for write: %s", filename);
	return FALSE;
    }    
    
    for (i = 0; file_formats[i].module_name; i++) 
    {
	if(!strncmp(file_formats[i].module_name, format, strlen(format)))
	    image_module = &(file_formats[i]);
    }
    
    if (!image_module) 
    {
	g_warning ("Unable to find handler for file: %s", filename);
	fclose (f);
	return FALSE;
    }
    
    if (image_module->module == NULL)
	gdk_pixbufsave_load_module (image_module);
    
    if (image_module->save == NULL) 
    {
	fclose (f);
	return FALSE;
    }
    
    (* image_module->save) (f, pixbuf, imginfo);
    fclose (f);
    return TRUE;
}
