/* io-jpg.c by David Welton <davidw@linuxcare.com>.  Most of the code
   borrowed from Imlib */

#include <stdio.h>
#include <jpeglib.h>
#include <setjmp.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "gdk-pixbuf-save.h"



#define DATABIG unsigned long long
#define DATA64  unsigned long long
#define DATA32  unsigned int
#define DATA16  unsigned short
#define DATA8   unsigned char

struct ImLib_JPEG_error_mgr
{
    struct jpeg_error_mgr   pub;
    sigjmp_buf              setjmp_buffer;
};

typedef struct ImLib_JPEG_error_mgr *emptr;
static void _JPEGFatalErrorHandler(j_common_ptr cinfo);
static void _JPEGErrorHandler(j_common_ptr cinfo);
static void _JPEGErrorHandler2(j_common_ptr cinfo, int msg_level);

static void
_JPEGFatalErrorHandler(j_common_ptr cinfo)
{
    emptr               errmgr;
    
    errmgr = (emptr) cinfo->err;
/*   cinfo->err->output_message(cinfo);*/
    siglongjmp(errmgr->setjmp_buffer, 1);
    return;
}

static void
_JPEGErrorHandler(j_common_ptr cinfo)
{
    emptr               errmgr;
   
    errmgr = (emptr) cinfo->err;
/*   cinfo->err->output_message(cinfo);*/
/*   siglongjmp(errmgr->setjmp_buffer, 1);*/
    return;
}

static void
_JPEGErrorHandler2(j_common_ptr cinfo, int msg_level)
{
    emptr               errmgr;
   
    errmgr = (emptr) cinfo->err;
/*   cinfo->err->output_message(cinfo);*/
/*   siglongjmp(errmgr->setjmp_buffer, 1);*/
    return;
    msg_level = 0;
}


int image_save(FILE *f, GdkPixbuf *pixbuf, GdkPixbufInfo *imginfo)
{
    struct              jpeg_compress_struct cinfo;
    DATA8              *buf;
    DATA8             *ptr;
    JSAMPROW           *jbuf;
    int                 y = 0, quality = imginfo->quality;
    int                 i, j;
    int                 w, h = 0;
    int rowstride       = 0;
    struct              ImLib_JPEG_error_mgr jerr;

    ArtPixBuf           *apixbuf;
	
    apixbuf = pixbuf->art_pixbuf;
    rowstride = apixbuf->rowstride;
    w = apixbuf->width;
    h = apixbuf->height;

    /* no image data? abort */
    if (!apixbuf->pixels)
	return 0;
    /* allocate a small buffer to convert image data */
    buf = malloc(w * 3 * sizeof(DATA8));
    if (!buf)
	return 0;

    /* set up error handling */
    jerr.pub.error_exit = _JPEGFatalErrorHandler;
    jerr.pub.emit_message = _JPEGErrorHandler2;
    jerr.pub.output_message = _JPEGErrorHandler;
    cinfo.err = jpeg_std_error(&(jerr.pub));
    if (sigsetjmp(jerr.setjmp_buffer, 1))
    {
	jpeg_destroy_compress(&cinfo);
	free(buf);
	fclose(f);
	return 1;
    }

    /* setup compress params */
    jpeg_create_compress(&cinfo);
    jpeg_stdio_dest(&cinfo, f);
    cinfo.image_width      = w;
    cinfo.image_height     = h;
    cinfo.input_components = 3; 
    cinfo.in_color_space   = JCS_RGB;
 
    /* look for tags attached to image to get extra parameters liek quality */
    /* settigns etc. - thsi si the "api" to hint for extra information for */
    /* saver modules */
/*     tag = __imlib_GetTag(im, "quality");
       if (tag)
       quality = tag->val;
       if (quality < 1)
       quality = 1;
       if (quality > 100)
       quality = 100;  */

    /* set up jepg compression parameters */
    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, quality, TRUE);
    jpeg_start_compress(&cinfo, TRUE);
    /* get the start pointer */
    ptr = apixbuf->pixels;
    /* go one scanline at a time... and save */
    i = 0;
    while (cinfo.next_scanline < cinfo.image_height)
    {
	/* convert scanline from ARGB to RGB packed */
 	for (j = 0; j < w; j++)
	    memcpy(&(buf[j*3]), &(ptr[i*rowstride + j*3]), 3);

	/* write scanline */
	jbuf = (JSAMPROW *)(&buf);
	jpeg_write_scanlines(&cinfo, jbuf, 1);
	i++;
	y++;
    }
    /* finish off */
    jpeg_finish_compress(&cinfo);   
    free(buf);
    return 0;
}
