/* io-png.c by David Welton <davidw@linuxcare.com>.  Most of the code
   borrowed from Imlib */

#include <stdlib.h>
#include <stdio.h>
#include <png.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "gdk-pixbuf-save.h"

#define DATABIG unsigned long long
#define DATA64  unsigned long long
#define DATA32  unsigned int
#define DATA16  unsigned short
#define DATA8   unsigned char

int image_save (FILE *f, GdkPixbuf *pixbuf, GdkPixbufInfo *imginfo)
{
    png_structp         png_ptr;
    png_infop           info_ptr;
    DATA8              *ptr;
    int                 x, y, j;
    png_bytep           row_ptr, data = NULL;
    png_color_8         sig_bit;
    int                 w, h, rowstride;
    ArtPixBuf          *apixbuf;
    int                 has_alpha;
    int                 bpc;

    apixbuf = pixbuf->art_pixbuf;
    bpc = apixbuf->bits_per_sample;
    w = apixbuf->width;
    h = apixbuf->height;
    rowstride = apixbuf->rowstride;
    has_alpha = apixbuf->has_alpha;

    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
				      NULL, NULL, NULL);
    if (!png_ptr)
    {
	fclose(f);
	return 0;
    }
    info_ptr = png_create_info_struct(png_ptr);
    if (info_ptr == NULL)
    {
	fclose(f);
	png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
	return 0;
    }
    if (setjmp(png_ptr->jmpbuf))
    {
	fclose(f);
	png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
	return 0;
    }
    png_init_io(png_ptr, f);
    if (has_alpha)
    {
	png_set_IHDR(png_ptr, info_ptr, w, h, bpc,
		     PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
#ifdef WORDS_BIGENDIAN
	png_set_swap_alpha(png_ptr);
#else
	png_set_bgr(png_ptr);
#endif
    }
    else
    {
	png_set_IHDR(png_ptr, info_ptr, w, h, bpc,
		     PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
	data = malloc(w * 3 * sizeof(char));
    }
    sig_bit.red = bpc;
    sig_bit.green = bpc;
    sig_bit.blue = bpc;
    sig_bit.alpha = bpc;
    png_set_sBIT(png_ptr, info_ptr, &sig_bit);
    png_write_info(png_ptr, info_ptr);
    png_set_shift(png_ptr, &sig_bit);
    png_set_packing(png_ptr);

    ptr = apixbuf->pixels;
    for (y = 0; y < h; y++)
    {
	if (has_alpha)
	    row_ptr = (png_bytep)ptr;
	else
	{
	    for (j = 0, x = 0; x < w; x++)
		memcpy(&(data[x*3]), &(ptr[x*3]), 3);

	    row_ptr = (png_bytep)data;
	}
	png_write_rows(png_ptr, &row_ptr, 1);
	ptr += rowstride;
    }
    if (data)
	free(data);
    png_write_end(png_ptr, info_ptr);
    png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
    return 0;
}
